#include <config.h>
#include <libecal/e-cal-view.h>

#include "koto-group-store.h"

G_DEFINE_TYPE (KotoGroupStore, koto_group_store, GTK_TYPE_LIST_STORE);

#define GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), KOTO_TYPE_GROUP_STORE, KotoGroupStorePrivate))

typedef struct _KotoGroupStorePrivate KotoGroupStorePrivate;

struct _KotoGroupStorePrivate
{
  ECalView *view;
  guint sig_added, sig_modified, sig_removed;
};

enum {
  PROP_NULL,
  PROP_CALVIEW,
};

/* TODO: optionally add none, seperator, group, separator, new.  The problem is
   that the group filter in the main window should have [All] and the groups,
   but the group selector in the editor should have [None], the groups, and [Add
   New]. */

/*
 * Private methods.
 */

static void
on_objects_added (ECalView *view, GList *objects, KotoGroupStore *store)
{
  for (;objects; objects = g_list_next (objects)) {
    icalcomponent *comp = objects->data;
    icalproperty *prop;
    char **categories, **l;
    
    prop = icalcomponent_get_first_property (comp, ICAL_CATEGORIES_PROPERTY);
    if (!prop)
      continue;

    categories = g_strsplit (icalproperty_get_categories (prop), ",", 0);

    for (l = categories; *l; l++) {
      GtkTreeIter iter;
      gtk_list_store_append (GTK_LIST_STORE (store), &iter);
      gtk_list_store_set (GTK_LIST_STORE (store), &iter,
                          COL_TYPE, TYPE_CATEGORY,
                          COL_NAME, *l, -1);
    }

    g_strfreev (categories);
    icalproperty_free (prop);
  }
}

static void
on_objects_modified (ECalView *view, GList *objects, KotoGroupStore *store)
{
  g_printerr ("TODO [%d objects modified]\n", g_list_length (objects));
}

static void
on_objects_removed (ECalView *view, GList *uids, KotoGroupStore *store)
{
  g_printerr ("TODO [%d objects deleted]\n", g_list_length (uids));
}


/*
 * Object methods.
 */

static void
koto_group_store_get_property (GObject *object, guint property_id,
                              GValue *value, GParamSpec *pspec)
{
  KotoGroupStorePrivate *priv = GET_PRIVATE (object);

  switch (property_id) {
  case PROP_CALVIEW:
    g_value_set_object (value, priv->view);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
  }
}

static void
koto_group_store_set_property (GObject *object, guint property_id,
                              const GValue *value, GParamSpec *pspec)
{
  switch (property_id) {
  case PROP_CALVIEW:
    koto_group_store_set_view (KOTO_GROUP_STORE (object),
                               g_value_get_object (value));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
  }
}

static void
koto_group_store_dispose (GObject *object)
{
  koto_group_store_set_view (KOTO_GROUP_STORE (object), NULL);

  if (G_OBJECT_CLASS (koto_group_store_parent_class)->dispose)
    G_OBJECT_CLASS (koto_group_store_parent_class)->dispose (object);
}

static void
koto_group_store_finalize (GObject *object)
{
  G_OBJECT_CLASS (koto_group_store_parent_class)->finalize (object);
}

static void
koto_group_store_class_init (KotoGroupStoreClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  g_type_class_add_private (klass, sizeof (KotoGroupStorePrivate));

  object_class->get_property = koto_group_store_get_property;
  object_class->set_property = koto_group_store_set_property;
  object_class->dispose = koto_group_store_dispose;
  object_class->finalize = koto_group_store_finalize;

  g_object_class_install_property (object_class, PROP_CALVIEW,
                                   g_param_spec_object ("calview", "calview", NULL,
                                                        E_TYPE_CAL_VIEW,
                                                        G_PARAM_READWRITE |
                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK));
}

static void
koto_group_store_init (KotoGroupStore *self)
{
  const GType column_types[] = { G_TYPE_INT, G_TYPE_STRING };

  gtk_list_store_set_column_types (GTK_LIST_STORE (self),
                                   G_N_ELEMENTS (column_types),
                                   (GType *) column_types);

  /* TODO: set sort function
  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self),
                                        GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
                                        GTK_SORT_ASCENDING);
  */
}


/*
 * Public methods.
 */

GtkTreeModel *
koto_group_store_new (ECalView *view)
{
  return g_object_new (KOTO_TYPE_GROUP_STORE,
                       "calview", view,
                       NULL);
}

void
koto_group_store_set_view (KotoGroupStore *store, ECalView *view)
{
  KotoGroupStorePrivate *priv;

  g_return_if_fail (KOTO_IS_GROUP_STORE (store));

  priv = GET_PRIVATE (store);

  if (priv->view) {
    g_signal_handler_disconnect (priv->view, priv->sig_added);
    g_signal_handler_disconnect (priv->view, priv->sig_modified);
    g_signal_handler_disconnect (priv->view, priv->sig_removed);
    g_object_unref (priv->view);
  }
  
  if (view) {
    priv->view = g_object_ref (view);
    priv->sig_added = g_signal_connect (priv->view, "objects_added", G_CALLBACK (on_objects_added), store);
    priv->sig_modified = g_signal_connect (priv->view, "objects_modified", G_CALLBACK (on_objects_modified), store);
    priv->sig_removed = g_signal_connect (priv->view, "objects_removed", G_CALLBACK (on_objects_removed), store);
  }
}
