#include <config.h>
#include "koto-task.h"

GType
koto_task_get_type (void)
{
  static GType type = 0;
  if (G_UNLIKELY (type == 0)) {
    type = g_boxed_type_register_static ("KotoTask",
                                         (GBoxedCopyFunc)koto_task_ref,
                                         (GBoxedFreeFunc)koto_task_unref);
  }
  return type;
}

KotoTask *
koto_task_new (icalcomponent *comp)
{
  KotoTask *task;

  g_return_val_if_fail (comp, NULL);
  
  task = g_slice_new (KotoTask);
  g_atomic_int_set (&task->ref_count, 1);
  task->comp = comp;

  return task;
}

KotoTask *
koto_task_ref (KotoTask *task)
{
  g_return_val_if_fail (task, NULL);
  g_return_val_if_fail (task->ref_count > 0, NULL);

  g_atomic_int_inc (&task->ref_count);

  return task;
}

void
koto_task_unref (KotoTask *task)
{
  g_return_if_fail (task);
  g_return_if_fail (task->ref_count > 0);

  if (g_atomic_int_dec_and_test (&task->ref_count)) {
    icalcomponent_free (task->comp);
    g_slice_free (KotoTask, task);
  }
}
